// **********************************************************************
//
// Copyright (c) 2003-2004 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <TestCommon.h>
#include <IceUtil/CtrlCHandler.h>
#include <IceUtil/Thread.h>
#include <IceUtil/Time.h>

#include <pthread.h>
#include <signal.h>


using namespace IceUtil;
using namespace std;

void callback(int signal)
{
    cout << "Handling signal " << signal << endl;
}

class TestThread : public Thread
{
public:

    TestThread(string name)
	: _name(name)
    {}

    virtual void run()
    {
	sigset_t nothing;
	sigemptyset(&nothing);
	
	sigset_t oldMask;
	sigemptyset(&oldMask);

	int rc = pthread_sigmask(SIG_BLOCK, &nothing, &oldMask);
	assert(rc == 0);

	if(sigismember(&oldMask, SIGHUP) == 1)
	{
	    cerr << _name << ": SIGHUP is blocked" << endl;
	}
	else
	{
	    cerr << _name  << ": SIGHUP is NOT blocked" << endl;
	}

	if(sigismember(&oldMask, SIGINT) == 1)
	{
	    cerr << _name  << ": SIGINT is blocked" << endl;
	}
	else
	{
	    cerr << _name  << ": SIGINT is NOT blocked" << endl;
	}

	if(sigismember(&oldMask, SIGTERM) == 1)
	{
	    cerr << _name  << ": SIGTERM is blocked" << endl;
	}
	else
	{
	    cerr << _name  << ": SIGTERM is NOT blocked" << endl;
	}
    }

private:
    string _name;
};


int main()
{
    ThreadPtr beforeTester = new TestThread("before");
    ThreadControl bt = beforeTester->start();
    CtrlCHandler handler(callback);
    
    ThreadPtr afterTester = new TestThread("after");
    ThreadControl at = afterTester->start();
    
    bt.join();
    at.join();
   
    return EXIT_SUCCESS;
}
